/*
 * chrome://tabmixplus/content/links/setup.js
 *
 * original code by Bradley Chapman
 * modified and developped by Hemiola SUN
 * modified again by Bradley Chapman
 *
 */
var gObserver =      Components.classes["@mozilla.org/observer-service;1"]
                     .getService(Components.interfaces.nsIObserverService);

var __contentAreaClick;
var __BrowserOpenTab;
var __BrowserLoadURL;
var __SearchLoadURL;
var __EMopenURL;
var __openUILink;

/**
 * @brief Install the link-handling functions of Tab Mix Plus.
 *
 * @returns		Nothing.
 */
function TMP_TBP_init()
{
   __contentAreaClick = window.contentAreaClick;
   __BrowserOpenTab = window.BrowserOpenTab;
   __BrowserLoadURL = window.BrowserLoadURL;
   if ("SearchLoadURL" in window) {
      __SearchLoadURL = window.SearchLoadURL;
      window.SearchLoadURL = TMP_SearchLoadURL;
   }
   __openUILink = window.openUILink;

  var eStrings = document.getElementById("extensionsStrings");
  if (eStrings) {
      // we're in the EM
      __EMopenURL = openURL;
      openURL = TMP_openURL;

      // catch call to tabmix options from EM
      eval("gExtensionsViewController.commands.cmd_options ="+gExtensionsViewController.commands.cmd_options.toString().replace(
      'var optionsURL = aSelectedItem.getAttribute("optionsURL");',
      '$& \ if (TMP_cmd_options(optionsURL)) return;'
      ));
  }

  // with MR Tech's local install
  if (typeof(Local_Install) == "object") {
    // use TMP call to TMP Options
    var _aURL = "'chrome://tabmixplus/content/pref/pref-tabmix.xul'";
    eval("Local_Install.createDropDownMenu ="+Local_Install.createDropDownMenu.toString().replace(
    'aMenuItem.setAttribute("oncommand", thisAction + "; event.stopPropagation();");',
    'if (thisAction.indexOf(_aURL) != -1) thisAction = "TMP_cmd_options(" + _aURL + ")"; \ $&'
    ));
  }

  window.contentAreaClick = TMP_contentAreaClick;
  window.BrowserOpenTab = TMP_BrowserOpenTab;
  window.BrowserLoadURL = TMP_BrowserLoadURL;
  window.openUILink = TMP_openUILink;
}

/**
 * @brief Remove the link-handling functions of Tab Mix Plus.
 *
 * @returns		Nothing.
 */
function TMP_TBP_exit()
{
    var eStrings = document.getElementById("extensionsStrings");
    if (eStrings) {
      // we're in the EM
      openURL = __EMopenURL;
    }

    window.BrowserOpenTab = __BrowserOpenTab;
    window.contentAreaClick = __contentAreaClick;
    window.BrowserLoadURL = __BrowserLoadURL;
    if ("SearchLoadURL" in window)
      window.SearchLoadURL = __SearchLoadURL;
    window.openUILink = __openUILink;

    return;
}

/**
 * @brief Force-call the window observer at least one time.
 *
 * @returns		Nothing.
 */
function TMP_TBP_Startup()
{
    // force-call the observer once, in order to kill new windows faster
    TMP_DOMWindowOpenObserver.onObserve(window, TMP_DOMWindowOpenObserver);

    // make tabmix compatible with ezsidebar
    var TMP_BrowserStartup = "BrowserStartup" in window ? "BrowserStartup" : "Startup";
    if ('__ezsidebar__'+TMP_BrowserStartup in window)
      TMP_BrowserStartup =  '__ezsidebar__'+TMP_BrowserStartup;

    if (!gPrefService)
         gPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                  .getService(Components.interfaces.nsIPrefBranch);

    var stringBundle = document.getElementById("tmp-string-bundle");
    var tabmix_loading = stringBundle.getString("session.loading.label") + "...";

    var windowOpeneByTabmix = "tabmixdata" in window;
    var firstWindow = numberOfWindows() == 1;

    var sessionStore = TMP_SessionStore.isSessionStoreEnabled() || "gSessionManager" in window;
    var sessionManager = gPref.getBoolPref("extensions.tabmix.sessions.manager");
    var crashRecovery = gPref.getBoolPref("extensions.tabmix.sessions.crashRecovery");
    var afterRestart = false;

    var restoreOrAsk = gPref.getIntPref("extensions.tabmix.sessions.onStart") < 2 || afterRestart;
    var afterCrash = gPref.prefHasUserValue("extensions.tabmix.sessions.crashed");
    if (afterCrash)
       gPref.clearUserPref("extensions.tabmix.sessions.crashed");

    // don't load home page on first window if session manager or crash recovery is enabled
    window.isLoadHomePage = true;
    if (!sessionStore && ((sessionManager && windowOpeneByTabmix) ||
         (firstWindow && crashRecovery && afterCrash) ||
         (firstWindow && sessionManager && restoreOrAsk))) {
            eval("window."+TMP_BrowserStartup+" ="+window[TMP_BrowserStartup].toString().replace(
              'uriToLoad = window.arguments[0];',
              'uriToLoad = gHomeButton.getHomePage() == window.arguments[0] ? "about:blank" : window.arguments[0];'
            ).replace(
              'var sidebarSplitter;',
              'if (uriToLoad && !gIsLoadingBlank) for (var i = 0; i < gBrowser.mTabContainer.childNodes.length ; i++) gBrowser.mTabContainer.childNodes[i].loadOnStartup = true;\
               if (uriToLoad == "about:blank" || "tabmixdata" in window) {\
               var aBrowser = gBrowser.mCurrentBrowser;\
               aBrowser.contentDocument.title = tabmix_loading;\
               aBrowser.mIconURL = "chrome://tabmixplus/skin/tmp.png";\
               gBrowser.mCurrentTab.setAttribute("image", aBrowser.mIconURL);}\
               var sidebarSplitter;'
            ));
    }

    var rExp = /BMSVC.readBookmarks\(\);|PlacesStarButton.init\(\);/
    eval("delayedStartup ="+delayedStartup.toString().replace(rExp,'$& \ TMP_delayedStartup();'));

    // look for installed extensions that are incompatible with tabmix
    if (gPref.getBoolPref("extensions.tabmix.disableIncompatible")) {
      if (typeof(PlacesController) == "function")
        window.setTimeout(disableExtensions, 3000, window);
      else
        disableExtensions();
    }

    // set the corect getTitleFromBookmark function
    window._getBookmarkTitle = gIsPlaces ? TMP_Places._getBookmarkTitle :
                                           TMP_Bookmark._getBookmarkTitle;

    // make tabmix compatible with menumanipulator and MenuEdit extensions
    // add bookmark options and tabmix function to context menu for tabs befor BrowserStartup
    // ***** we dno't need this in Firefox 3.0 *****
    window.gNavigatorBundle = window.document.getElementById("bundle_browser");
    if(window.addBookmarkMenuitems) {
      window.addBookmarkMenuitems();
      window.addBookmarkMenuitems = function(){}
    }
    if(window.AugmentTabs) {
      window.AugmentTabs.init = function(){}
    }
    window.TM_creatMenuItems();

    // if nglayout.debug.disable_xul_cache == true sometimes sessionHistory act strange
    // especially with many extensions installed
    var pref = "nglayout.debug.disable_xul_cache";
    var openFromPlaces;
    if ("openFromPlaces" in window) {
      openFromPlaces = window.openFromPlaces;
      delete window.openFromPlaces;
    }

    if ((!firstWindow && !windowOpeneByTabmix || openFromPlaces) && gPref.prefHasUserValue(pref) && gPref.getBoolPref(pref))
      window.setTimeout(window[TMP_BrowserStartup], 0);
    else
      window[TMP_BrowserStartup]();

    return;
}

function getTitleFromBookmark(aUrl, aTitle)
{
   if (!gPref.getBoolPref("extensions.tabmix.titlefrombookmark") || !aUrl)
      return aTitle;

   var title = _getBookmarkTitle(aUrl);
   if (title || !window.gIeTab)
      return title || aTitle;

   // IE Tab is installed try to find url with or without the prefix
   var ietab = "chrome://ietab/content/reloaded.html?url="
   if (aUrl == ietab)
      return aTitle;
   if (aUrl.indexOf(ietab) == 0)
      title = _getBookmarkTitle(aUrl.replace(ietab, ""));
   else
      title = _getBookmarkTitle(ietab + aUrl);

   return title || aTitle;
}

function TMP_delayedStartup()
{
  if (numberOfWindows() == 1) {
    // make sure that we don't enable both sessionStore and session manager
    TMP_SessionStore.setService(1, true);
    if (TMP_SessionStore.isSessionStoreEnabled()) {
      // Add attribute to nsSessionStore persistTabAttribute after delay
      // we call this after nsSessionStore.init
      window.setTimeout(TMP_SessionStore.persistTabAttribute, 2000);
    }
  }

  // set title at startup if we not use session manager
  // startup page or home page load befor BMSVC
  if (gPrefService.getBoolPref("extensions.tabmix.titlefrombookmark")) {
    for (var i = 0; i < gBrowser.mPanelContainer.childNodes.length ; i++) {
      var aBrowser = gBrowser.getBrowserAtIndex(i);
      var aUrl = aBrowser.contentDocument.baseURI;
      aUrl = (aUrl) ? aUrl : aBrowser.currentURI.spec ;
      var bookMarkName = getTitleFromBookmark(aUrl);
      if (bookMarkName && aBrowser.contentDocument.title != bookMarkName)
        aBrowser.contentDocument.title = bookMarkName;
    }
  }
}

// this must run befor all
function TMP_beforStartup(tabBrowser)
{
    // fix conflict with Lori extension
    // and fix bug in Lori extension that prevent it from closing unread tabs
    if ("loriOldRemoveTab" in tabBrowser) {
      tabBrowser.removeTab = tabBrowser.loriOldRemoveTab;
      delete tabBrowser.loriOldRemoveTab;
      eval("tabBrowser.removeTab ="+tabBrowser.removeTab.toString().replace(
         'var index = -1;',
         'var b = this.getBrowserForTab(aTab); \
          if ("loriTabListener" in b) { \
            b.loriTabListener.destroy(); \
            b.loriTabListener = null; \
          } \ $&'
      ));
    }


    if (typeof(BrowserOpenAddonsMgr) != "function")
        return;

    // remove call to "arrowscrollbox" by "this.mTabContainer.mTabstrip" we don't use it in tabmix
    // for Firefox 2.0 +
    eval("tabBrowser.moveTabTo ="+tabBrowser.moveTabTo.toString().replace(
         'this.mTabContainer.insertBefore(aTab, this.mTabContainer.childNodes[aIndex]);',
         'this.mTabContainer.insertBefore(aTab, this.mTabContainer.childNodes.item(aIndex));'
        ).replace(
         'this.mTabContainer.mTabstrip.scrollBoxObject.ensureElementIsVisible(this.mCurrentTab);',
         ''
    ));

    eval("tabBrowser.addTab ="+tabBrowser.addTab.toString().replace(
         'aTabContainer.mTabstrip._updateScrollButtonsDisabledState();',
         ''
        ).replace(
         'this.mTabContainer.mTabstrip.scrollBoxObject.scrollBy(this.mTabContainer.firstChild.boxObject.width, 0);',
         ''
    ));

    eval("tabBrowser.removeTab ="+tabBrowser.removeTab.toString().replace(
         'setTimeout(_delayedUpdate, 0, this.mTabContainer);',
         ''
        ).replace(
         'tabStrip.scrollBoxObject.getPosition(scrollPos, {});',
         'scrollPos.value = -tabStrip.boxObject.width;'
        ).replace(
         'tabStrip.scrollBoxObject.getScrolledSize(scrolledSize, {});',
         'scrolledSize.value = 2000;'
    ));

    eval("tabBrowser.mTabContainer.handleEvent ="+tabBrowser.mTabContainer.handleEvent.toString().replace(
         'this.mTabstrip.scrollBoxObject.ensureElementIsVisible(this.selectedItem);',
         'this.ensureTabIsVisible(this.selectedIndex);'
         ).replace( // for Firefox 3.0 after bug 347363
         'case "resize":',
         'case "resize": break;'
    ));

// return true if all tabs in the window are blank
tabBrowser.isBlankWindow = function()
{
   for (var i = 0; i < this.mTabs.length; i++)
      if (!this.isBlankBrowser(this.getBrowserAtIndex(i))) return false;
   return true;
}

tabBrowser.isBlankTab = function(aTab)
{
   return this.isBlankBrowser(this.getBrowserForTab(aTab));
}

tabBrowser.isBlankNotBusyTab = function TMP_isBlankNotBusyTab(aTab)
{
   if (aTab.hasAttribute("busy"))
      return false;

   return this.isBlankBrowser(this.getBrowserForTab(aTab));
}

tabBrowser.isBlankBrowser = function _isBlankBrowser(aBrowser)
{
   try{
      if (!aBrowser) return true;
      return (!aBrowser.sessionHistory || aBrowser.sessionHistory.index < 0 ||
         (aBrowser.sessionHistory.count < 2 &&
            (!aBrowser.currentURI || aBrowser.currentURI.spec == 'about:blank')));
   } catch (e) {tmLog(e); return true;}
}

    if (!gIsFirefox3) {
      var _notifyBackgroundTab = tabBrowser.mTabContainer._notifyBackgroundTab.toString().split("(tsboStart > ctboStart || ctboEnd > tsboEnd)")
      _notifyBackgroundTab[0] = "function _notifyBackgroundTab(aTab) { \n if ";
      _notifyBackgroundTab = _notifyBackgroundTab.join("(!this.isTabVisible(aTab._tPos))");
      eval("tabBrowser.mTabContainer._notifyBackgroundTab =" + _notifyBackgroundTab);
    }

    var _mAllTabsPopup = tabBrowser.mTabContainer.mAllTabsPopup;
    if ("_updateTabsVisibilityStatus" in _mAllTabsPopup)
      eval("_mAllTabsPopup._updateTabsVisibilityStatus ="+_mAllTabsPopup._updateTabsVisibilityStatus.toString().replace(
         'var tabstripBO = tabContainer.mTabstrip.scrollBoxObject;',
         'var tabstripBO = tabContainer.tabstrip.boxObject;'
      ));

   /*
    * Session Manager extesion add tabs too soon for us to check isTabVisible properly
    * we get wrong scrollstatus at startup
    * we add flag to use in tabBrowser.mTabContainer.isTabVisible
    */
    if ("gSessionManager" in window) {
       var ss = Cc["@mozilla.org/browser/sessionstartup;1"].getService(Ci.nsISessionStartup);
       if (ss.doRestore()) {
          tabBrowser.mTabContainer.setAttribute("SM_restart", true);
          window.addEventListener("load", function() {
             window.removeEventListener("load", arguments.callee, false);
             setTimeout(function (tabContainer) { tabContainer.removeAttribute("SM_restart"); }, 0, tabBrowser.mTabContainer);
          }, false);
       }
    }

    // add event for update tab when restoring from nsSessionStore
    tabBrowser.mTabContainer.addEventListener("SSTabRestoring", TMP_SSTabRestoring, true);

    // add event for creating a new tab
    tabBrowser.mTabContainer.addEventListener("DOMNodeInserted", tabxTabAdded, true);

    tabBrowser.mTabContainer.addEventListener("TabClose", tabxTabClosed, true);

    // add event for mouse scrolling on tab bar, necessary for linux
    tabBrowser.mTabContainer.addEventListener("DOMMouseScroll", TMtabBarScroll, false);
    if (/^Linux/.test(navigator.platform))
       document.getElementById("navigator-toolbox").addEventListener("DOMMouseScroll", TMtabBarScroll, false);

}
