/*
 * chrome://tabmixplus/content/session/sessionStore.js
 *
 * original code by onemen
 *
 */

var TMP_SessionStore = {
   /**
    * @brief       - Add attribute to nsSessionStore persistTabAttribute.
    *
    * @param doInit   a Boolean value - true when we need to init nsISessionStore.
    *
    * @returns        Nothing.
    */
   persistTabAttribute: function TMP_ss_persistTabAttribute() {
      var aTab = gBrowser.mTabContainer.firstChild;
      if (aTab != gBrowser.mCurrentTab) {
         aTab.removeAttribute("selected");
         aTab.removeAttribute("flst_id");
      };

      try {
        /*
         * XUL Tab attributes to (re)store
         * Restored in nsSessionStore restoreHistory()
         */
         var _xulAttributes = ["image", "protected", "_locked", "fixed-label", "label-uri", "reload-data"];

         var ss = Cc["@mozilla.org/browser/sessionstore;1"].getService(Ci.nsISessionStore);
         _xulAttributes.forEach(function(aAttr) {
            ss.persistTabAttribute(aAttr);
         });
      } catch(ex) {
         tmLog("nsSessionStore could not add Attribute to persistTabAttribute: " + ex + "\n");
      }
   },

   /**
    * @brief         make sure that we don't enable both sessionStore and session manager
    *
    * @param msgNo   a Integer value - msg no. to show.
    *
    * @param start   a Boolean value - true if we call this function befor startup.
    *
    * @returns       Nothing.
    */
   setService: function TMP_ss_setSessionService(msgNo, start, win) {
      if(window.setSession || gPref.prefHasUserValue("extensions.tabmix.setDefault"))
         return;

     /*
      * From 2008-03-10 we don't set browser.sessionstore.enabled to false anymore
      * we use nsISessionStore service in TMP.
      * if we use TMP session manager we set all other sessionstore pref to false disableSessionRestore
      * if both TMP session manager and browser.sessionstore.enabled is false we don't do anything at the moment
      */
      var TMP_sessionManager_enabled = gPref.getBoolPref("extensions.tabmix.sessions.manager") ||
                                     gPref.getBoolPref("extensions.tabmix.sessions.crashRecovery");
      if (!TMP_sessionManager_enabled)
         return;

      window.setSession = true;
      // if session manager extension is install disable TMP session manager
      if ("gSessionManager" in window) {
         // session manager extension take care about nsISessionStore service pref
         gPref.setBoolPref("extensions.tabmix.sessions.manager", false);
         gPref.setBoolPref("extensions.tabmix.sessions.crashRecovery", false);
      }
      else {
         if (!gPref.getBoolPref("browser.sessionstore.enabled")) {
            // TMP is the session manager make sure nsISessionStore service is enabled
            // and disable all other sessionstore pref
            gPref.setBoolPref("browser.sessionstore.enabled", true);
            this.setSessionRestore(false);
         }
         else if (this.isSessionStoreEnabled()) {
            // ask the user to choose between TMP session manager and sessionstore
            var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                                     .getService(Ci.nsIPromptService);
            var bundle_session = document.getElementById("bundle_session_manager");

            var msg = start ? bundle_session.getString("sm.disable.msg") + "\n\n" : "";
            msg += bundle_session.getString("sm.disable.msg" + msgNo)
            var buttonPressed = promptService.confirmEx(win || window,
                           "TabMix " + bundle_session.getString("sm.title"),
                           msg,
                           (promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0)
                           + (promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1),
                           null, null, null, null, {});

            if ((msgNo == 1 && buttonPressed == 1) || ((msgNo == 2 && buttonPressed == 0)))
               this.setSessionRestore(false);
            else {
               // we don't chane any of sessionstore default setting
               // the user will be ask on exit what to do. (browser.warnOnRestart and browser.warnOnQuit are both true on default)
               gPref.setBoolPref("extensions.tabmix.sessions.manager", false);
               gPref.setBoolPref("extensions.tabmix.sessions.crashRecovery", false);
            }
         }
      }

      delete window.setSession;
   },

   isSessionStoreEnabled: function () {
     return gPref.getBoolPref("browser.sessionstore.enabled") &&
        (gPref.getIntPref("browser.startup.page") == 3 || gPref.getBoolPref("browser.sessionstore.resume_from_crash"));
   },

   isAfterRestart: function () {
     if (gPref.getBoolPref("browser.sessionstore.enabled")) {
       var ss = Cc["@mozilla.org/browser/sessionstartup;1"].
                      getService(Ci.nsISessionStartup);
       // when TMP session manager is enabled this is true only after restart
       return ss.doRestore();
     }
     return false;
   },

   setSessionRestore: function (aEnable) {
      gPref.setBoolPref("browser.warnOnRestart", aEnable);
      gPref.setBoolPref("browser.warnOnQuit", aEnable);
      gPref.setBoolPref("browser.sessionstore.resume_from_crash", aEnable);
      gPref.setIntPref("browser.startup.page", aEnable ? 3 : 1);
   },

   /**
    * @brief           remove closed tab from sessionStore list without restore
    *
    * @param aIndex    a Integer value - 0 or grater index to remove
    *                  other value empty the list.
    *
    * @returns         Nothing.
    */
   removeClosedTab: function TMP_ss_removeClosedTab(aIndex) {
      if (!gIsFirefox2 || !window.__SSi)
         return;

      var ss = Cc["@mozilla.org/browser/sessionstore;1"].
                  getService(Ci.nsISessionStore);

      var state = { windows: [], _firstTabs: true };
      state.windows[0] = { _closedTabs: [] };

      if (aIndex >= 0) {
         // get closed-tabs from nsSessionStore
         var closedTabs = eval("(" + ss.getClosedTabData(window) + ")");
         // purge closed tab at aIndex
         closedTabs.splice(aIndex, 1);
         state.windows[0]._closedTabs = closedTabs;
      }

      // replace existing _closedTabs
      ss.setWindowState(window, state.toSource(), false);
   },

   /**
    * @brief           get attribute xultab data
    *
    * @param xultab    an object value - xultab from nsSessionStore
    *
    * @param attrib    attribute name as string
    *
    * @returns         attribute value as string or empty string.
    */
   getAttribute: function TMP_ss_getAttribute(xultab, attrib) {
      if (!xultab)
         return "";
      var _data = xultab.split(" ");
      for ( var i= 0; i < _data.length; i++ ){
         if (/^([^\s=]+)=(.*)/.test(_data[i]) && RegExp.$1 == attrib)
            return decodeURI(RegExp.$2);
      }
      return  "";
   }
}

/* ::::::::::::::::::::::::::::::::::::::::::::::::::::: */

// get closed-tabs from nsSessionStore
function getClosedTabs() {
  if (!TMP_SessionStore.isSessionStoreEnabled())
    return;

  var ss = Cc["@mozilla.org/browser/sessionstore;1"].
           getService(Ci.nsISessionStore);

  if (ss.getClosedTabCount(window) == 0)
    return;

  var undoItems = eval("(" + ss.getClosedTabData(window) + ")");
  var tab, tabData;
  for (var i = undoItems.length - 1; i >= 0 ; i--) {
    tab = undoItems[i];
    tabData = tab.state;

    var history = Components.classes["@mozilla.org/browser/shistory;1"]
                                 .createInstance(tmSHistory);
    history = history.QueryInterface(Components.interfaces.nsISHistoryInternal);
    var aIdMap = { used: {} };

    for (var j in tabData.entries)
      history.addEntry(TMP_deserializeHistoryEntry(tabData.entries[j], aIdMap), true);

    // set history index
    var activeIndex = (tabData.index || tabData.entries.length) - 1;
    var _blank =  history.getEntryAtIndex(activeIndex, true);

    var scrollPos = (tabData.entries[activeIndex].scroll || "0,0").split(",");
    scrollPos = [parseInt(scrollPos[0]) || 0, parseInt(scrollPos[1]) || 0];

    gBrowser.closedTabs.push([tab.pos,
                               history,
                               tab.title,
                               TMP_SessionStore.getAttribute(tabData.xultab, "image"),
                               "0" + (TMP_SessionStore.getAttribute(tabData.xultab, "_locked") != "false" ? "1" : "0") + "11111",
                               scrollPos[0] + "," + scrollPos[1] + "," + parseFloat(tabData.zoom || 1)
                              ]);

   if (gBrowser.closedTabs.length > gPref.getIntPref("browser.sessionstore.max_tabs_undo"))
      gBrowser.closedTabs.shift();
  }

  TMP_ClosedTabs.setButtonDisableState(gBrowser.closedTabs.length == 0);
}

  function TMP_deserializeHistoryEntry(aEntry, aIdMap) {
    var shEntry = Cc["@mozilla.org/browser/session-history-entry;1"].
                  createInstance(Ci.nsISHEntry);

    var ioService = Cc["@mozilla.org/network/io-service;1"].
                    getService(Ci.nsIIOService);
    shEntry.setURI(ioService.newURI(aEntry.url, null, null));
    shEntry.setTitle(aEntry.title || aEntry.url);
    shEntry.setIsSubFrame(aEntry.subframe || false);
    shEntry.loadType = Ci.nsIDocShellLoadInfo.loadHistory;

    if (aEntry.cacheKey) {
      var cacheKey = Cc["@mozilla.org/supports-PRUint32;1"].
                     createInstance(Ci.nsISupportsPRUint32);
      cacheKey.data = aEntry.cacheKey;
      shEntry.cacheKey = cacheKey;
    }
    if (aEntry.ID) {
      // get a new unique ID for this frame (since the one from the last
      // start might already be in use)
      var id = aIdMap[aEntry.ID] || 0;
      if (!id) {
        for (id = Date.now(); aIdMap.used[id]; id++);
        aIdMap[aEntry.ID] = id;
        aIdMap.used[id] = true;
      }
      shEntry.ID = id;
    }

    var scrollPos = (aEntry.scroll || "0,0").split(",");
    scrollPos = [parseInt(scrollPos[0]) || 0, parseInt(scrollPos[1]) || 0];
    shEntry.setScrollPosition(scrollPos[0], scrollPos[1]);

    if (aEntry.postdata) {
      var stream = Cc["@mozilla.org/io/string-input-stream;1"].
                   createInstance(Ci.nsIStringInputStream);
      stream.setData(aEntry.postdata, -1);
      shEntry.postData = stream;
    }

    if (aEntry.children && shEntry instanceof Ci.nsISHContainer) {
      for (var i in aEntry.children) {
        shEntry.AddChild(TMP_deserializeHistoryEntry(aEntry.children[i], aIdMap), i);
      }
    }

    return shEntry;
  }
/* ::::::::::::::::::::::::::::::::::::::::::::::::::::: */

var TMP_ClosedTabs = {
   _buttonBroadcaster: null,
   get buttonBroadcaster() {
      if (!this._buttonBroadcaster)
        this._buttonBroadcaster = document.getElementById("tmp_undocloseButton");
      return this._buttonBroadcaster;
   },

   /**
    * check to see if nsISessionStore is initialized
    */
   get ssIsON() {
      return gIsFirefox2 && "__SSi" in window;
   },

   // make btn_undoclose single-functionality or dual-functionality
   setButtonType: function(menuOnly) {
      var buttonType = menuOnly ? "menu" : "menu-button";
      if (this.buttonBroadcaster.getAttribute("type") != buttonType)
         this.buttonBroadcaster.setAttribute("type", buttonType);
   },

   setButtonDisableState: function ct_setButtonDisableState(value) {
      if (!value)
         this.buttonBroadcaster.removeAttribute("disabled");
      else if (this.buttonBroadcaster.getAttribute("disabled") != "true")
         this.buttonBroadcaster.setAttribute("disabled", "true");
   },

   _ss: null,
   get ss() {
      if (!this._ss)
         this._ss = Cc["@mozilla.org/browser/sessionstore;1"].
                     getService(Ci.nsISessionStore);

      return this._ss;
   },

  /**
    * Get closed tabs count
    */
   get count() {
      if (this.ssIsON)
         return this.ss.getClosedTabCount(window);
      else
         return this._closedTabs.length;
   },

  /**
    * Get closed tabs data
    */
   get getClosedTabData() {
      if (this.ssIsON)
         return eval("(" + this.ss.getClosedTabData(window) + ")")
      else
         return gBrowser.closedTabs;
   },

  /**
   * @brief           fetch the data of closed tab, while removing it from the array
   *
   * @param aIndex    a Integer value - 0 or grater index to remove
   *                  other value empty the list.
   *
   * @returns         close tab data at aIndex.
   */
   getClosedTabAtIndex: function ct_getClosedTabAtIndex(aIndex) {
      var closedTabs = this.getClosedTabData;
      var closedTab;
      var state = { windows: [], _firstTabs: true };
      state.windows[0] = { _closedTabs: [] };
      // if aIndex is not > 0 we just past empy list to setWindowState
      // it's like remove all closed tabs from the list
      if (aIndex >= 0) {
         // purge closed tab at aIndex
         closedTab = closedTabs.splice(aIndex, 1).shift();
         state.windows[0]._closedTabs = closedTabs;
      }

      // replace existing _closedTabs
      this.ss.setWindowState(window, state.toSource(), false);

      // remove the tab from tabmix list
      var index = gBrowser.closedTabs.length - 1 - aIndex;
      gBrowser.closedTabs.splice(index, 1).shift();
      var updateRDF = SessionManager.enableBackup && SessionPref.getBoolPref("save.closedtabs");
      if (updateRDF)
         SessionManager.deleteClosedtabAt(index + 1);

      this.setButtonDisableState(this.count == 0);
      return closedTab;
   },

   SSS_undoCloseTab: function ct_SSS_undoCloseTab(aIndex, aWhere, aSelectRestoredTab, aTabToRemove) {
      if (!tabxPrefs.getBoolPref("undoClose") || this.count == 0)
         return null;

      // get tab data
      var tabData = this.getClosedTabAtIndex(aIndex);

      var cTab = gBrowser.mCurrentTab;
      if (aWhere == "current") {
         cTab.collapsed = true;
         aTabToRemove = cTab;
         tabData.pos = cTab._tPos;
      }
      else if (typeof(aTabToRemove) == "undefined" && gBrowser.isBlankNotBusyTab(cTab))
         aTabToRemove = cTab;

      // add restored tab to current window
      var newTab;
      if (gIsFirefox3) {
         newTab = gBrowser.addTab();
         this.ss.setTabState(newTab, tabData.state.toSource());
      }
      else { // there is no setTabState/getTabState in firefox 2.0
        var state = { windows: [ { tabs: [tabData.state] } ] };
        this.ss.setWindowState(window, state.toSource(), false);
        newTab = gBrowser.getTabForLastPanel();
      }

      // after we open new tab we only need to fix position if this is true
      var restorePosition = tabxPrefs.getBoolPref("undoClosePosition");
      if ( aWhere == "current" || (aWhere == "original" && restorePosition) ) {
         gBrowser.TMmoveTabTo(newTab, Math.min(gBrowser.mTabs.length - 1, tabData.pos), 1);
      }
      else if (aWhere != "end")
         // we don't call TMP_openTabNext from add tab if it called from sss_undoCloseTab
         gBrowser.TMP_openTabNext(newTab);

      if (aSelectRestoredTab) {
         content.focus();
         gBrowser.TMP_selectNewForegroundTab(newTab, false, null, false);
      }

      if (aTabToRemove)
         gBrowser.removeTab(aTabToRemove);

      return newTab;
   },

   // for replace browser.js undoCloseTab
   window_undoCloseTab: function ct_window_undoCloseTab(aIndex, aWhere) {
      if (TMP_ClosedTabs.ssIsON)
         TMP_ClosedTabs.SSS_undoCloseTab(aIndex || 0, aWhere || "original", true);
      else
         gBrowser.undoRemoveTab();
   },

   undoCloseTab: function ct_undoCloseTab(aIndex) {
      if (this.ssIsON)
         this.SSS_undoCloseTab(aIndex || 0, "original", true);
      else
         gBrowser.undoRemoveTab();
   }

}

var convertSession = {
   get _prompt() {
      return Cc["@mozilla.org/embedcomp/prompt-service;1"]
                            .getService(Ci.nsIPromptService);
   },

   get getTitle() {
      return document.getElementById("tmp-string-bundle").getString("incompatible.title") + " - " +
                document.getElementById("bundle_session_manager").getString("sm.title");
   },

   getString: function (aEntity) {
      return document.getElementById("bundle_session_manager").
         getString("sm.extension.convert." + aEntity);
   },

   startup: function () {
      var isFirstWindow = "gSessionManager" in window && numberOfWindows() == 1 && !window.duplicat;
      if (!isFirstWindow)
         return;

      var sessions = SessionManager.getSessionList();
      if (!sessions)
         return;

      if(SessionManager.nodeHasArc("rdf:gSessionManager", "status"))
         return;

      SessionManager.setLiteral("rdf:gSessionManager", "status", "converted");
      SessionManager.dataFlush();

      var rv = this.confirm(this.getString("msg1") + "\n\n" + this.getString("msg2"));
      if (rv == 0)
         this.doConvert(sessions);
   },

   convertFile: function (aFileUri) {
      var sessions;
      if (aFileUri) {
         try {
            var tmpDATASource = SessionManager.DATASource;
            SessionManager.DATASource = SessionManager.RDFService.GetDataSourceBlocking(aFileUri);
            sessions = SessionManager.getSessionList();
         } catch (e) { // corrupted session.rdf
            SessionManager.DATASource = tmpDATASource;
            tmLog(e);
         }
      }
      else
         sessions = SessionManager.getSessionList();

      var msg;
      if (!sessions) {
         this._prompt.alert(null, this.getTitle, this.getString("nosessions"));
         return;
      }
      var rv = 0;
      if(SessionManager.nodeHasArc("rdf:gSessionManager", "status")) {
         rv = this.confirm(this.getString("alreadyconverted") + "\n\n" + this.getString("doitagain"));
      }
      else {
         SessionManager.setLiteral("rdf:gSessionManager", "status", "converted");
         SessionManager.dataFlush();
      }
      if (rv == 0)
         this.doConvert(sessions);

      if (tmpDATASource)
         SessionManager.DATASource = tmpDATASource;
   },

   confirm: function (aMsg) {
      var promptService = this._prompt;
      return promptService.confirmEx(null,
                           this.getTitle,
                           aMsg,
                           (promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0)
                           + (promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1),
                           null, null, null, null, {});
   },

   doConvert: function (sessions) {
      window.sessionConvertTimestamp = new Date().valueOf() - 100;

      var sessionsPath = sessions.path.push(gSessionPath[3]);
      var sessionsName = sessions.list.push("Crashed Session");
      var _count = 0;

      for (var i = 0; i < sessions.path.length; i++ ) {
         var sessionState = this.getSessionState(sessions.path[i]);

         // get timestamp from nameExt property
         var dateString = "", fileDate;
         var nameExt = SessionManager.getLiteralValue(sessions.path[i], "nameExt");
         if (nameExt) {
            var date = nameExt.substr(nameExt.length - 20, 10);
            var time = nameExt.substr(nameExt.length - 9, 8);
            fileDate = " (" + date.split("/").join("-") + ")";
            dateString = " (" + date.split("/").join("-") + " " + time.substr(0, time.length - 3) + ")";
            var _time = time.split(":");
            var timestamp = new Date(date).valueOf() + 3600*_time[0] + 60*_time[1] + 1*_time[2];
         }
         var sessionName = unescape(sessions.list[i]);
         var name = "[ Tabmix ] " + sessionName + dateString;
         var fileName = gSessionManager.makeFileName("Tabmix - " + sessionName + fileDate);

         _count += this.save(sessionState, timestamp, name, fileName);
      }

      var msg;
      if (_count == 0) {
         this._prompt.alert(null, this.getTitle, this.getString("unable"));
         return;
      }
      else if (_count > 1)
         msg = _count + " " + this.getString("many") + "\n" + this.getString("rename");
      else
         msg = this.getString("one") + "\n" + this.getString("rename");

      var session = gSessionManager.selectSession(msg, gSessionManager._string("remove_session_ok"), { multiSelect: true, remove: true });
      delete window.sessionConvertTimestamp;
      if (session)
         gSessionManager.remove(session);
   },

   eval_getSessions: function () {
      var wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
                        .getService(Components.interfaces.nsIWindowMediator);
      var win = wm.getMostRecentWindow("navigator:browser");

      if (win && win.sessionConvertTimestamp) {
         eval("gSessionManager.getSessions ="+gSessionManager.getSessions.toString().replace(
             'var timestamp = parseInt(RegExp.$2) || file.lastModifiedTime;',
             'if (file.lastModifiedTime < ' + win.sessionConvertTimestamp + ') continue; \ $&'
         ).replace(
             'sessions.latestName = RegExp.$1;',
             ''
         ));
      }
   },

   save: function (aSession, aTimestamp, aName, aFileName) {
      if (aSession.windows.length == 0)
         return false;

      if (!aSession.session)
         aSession.session = { state:"stop" };
      var oState = "[SessionManager]\nname=" + aName + "\ntimestamp=" + aTimestamp + "\n" + aSession.toSource();
      var file = gSessionManager.getSessionDir(gSessionManager.makeFileName(aName));
      try {
         var file = gSessionManager.getSessionDir(aFileName, true);
         gSessionManager.writeFile(file, oState);
      }
      catch (ex) {
         tmLog(ex);
         return false;
      }
      return true;
   },

   getSessionState: function (aPath) {
      var _windows = [];
      var sessionEnum = SessionManager.initContainer(aPath).GetElements();
      while (sessionEnum.hasMoreElements()) {
         var rdfNodeWindow = sessionEnum.getNext();
         if (rdfNodeWindow instanceof tmRDFResource) {
            var windowPath = rdfNodeWindow.QueryInterface(tmRDFResource).Value;
            if (SessionManager.nodeHasArc(windowPath, "dontLoad"))
               continue;
            var aWindowState = this.getWindowState(rdfNodeWindow);
            if (aWindowState) // don't save empty windows
               _windows.push(aWindowState);
         }
      }
      return { windows: _windows };
   },

   getWindowState: function (rdfNodeWindow) {
      var state = { tabs: [], selected: 0, _closedTabs: [] };

      var rdfNodeTabs = SessionManager.getResource(rdfNodeWindow, "tabs");
      if (!(rdfNodeTabs instanceof tmRDFResource) || SessionManager.containerEmpty(rdfNodeTabs)) {
         return null;
      }
      state.tabs = this.getTabsState(rdfNodeTabs);
      state._closedTabs = this.getClosedTabsState(SessionManager.getResource(rdfNodeWindow, "closedtabs"));
      state.selected = SessionManager.getIntValue(rdfNodeWindow, "selectedIndex");
      return state;
   },

   getTabsState: function (rdfNodeTabs) {
      var _tabs = [];
      var tabsEnum = SessionManager.initContainer(rdfNodeTabs).GetElements();
      while (tabsEnum.hasMoreElements()) {
         var rdfNodeTab = tabsEnum.getNext();
         if (rdfNodeTab instanceof tmRDFResource)
            _tabs.push(this.getTabState(rdfNodeTab));
      }
      return _tabs;
   },

   getClosedTabsState: function (rdfNodeTabs) {
      var _tabs = [];
      var tabsEnum = SessionManager.initContainer(rdfNodeTabs).GetElements();
      while (tabsEnum.hasMoreElements()) {
         var rdfNodeTab = tabsEnum.getNext();
         if (rdfNodeTab instanceof tmRDFResource) {
            var closedTab = {};
            closedTab.state = this.getTabState(rdfNodeTab);
            closedTab.title = closedTab.state.entries[0].title;
            closedTab.pos = SessionManager.getIntValue(rdfNodeTab, "tabPos");
            _tabs.push(closedTab);
         }
      }
      return _tabs;
   },

   getTabState: function (rdfNodeTab) {
      var tabData = {entries:[], index: 0, zoom: 1, disallow:"", xultab: "", extData: null, text:""};
      tabData.entries = this.getHistoryState(rdfNodeTab);
      tabData.index = SessionManager.getIntValue(rdfNodeTab, "index");
      tabData.zoom = SessionManager.getLiteralValue(rdfNodeTab, "scroll").split(",")[2];
      var tabProperties = SessionManager.getLiteralValue(rdfNodeTab, "properties");
      var tabAttribute = ["Images","Subframes","MetaRedirects","Plugins","Javascript"];
      var disallow = [];
      for (var j = 2; j < tabProperties.length; j++ ) {
         if (tabProperties.charAt(j) != "1")
            disallow.push(tabAttribute[j]);
      }
      tabData.disallow = disallow.join(",");
      var xultab = [];
      var image = SessionManager.getLiteralValue(rdfNodeTab, "image");
      if (image)
         xultab.push("image=" + image);
      if (tabProperties.charAt(0) == "1")
         xultab.push("protected=true");
      xultab.push("_locked=" + (tabProperties.charAt(1) == "1"));
      tabData.xultab = xultab.join(" ");
      return tabData;
   },

   getHistoryState: function (rdfNodeTab) {
      var tmpData = SessionManager.getLiteralValue(rdfNodeTab, "history").split("|-|");
      var sep = tmpData.shift(); // remove seperator from data
      var historyData = tmpData.join("|-|").split(sep);
      var historyCount = historyData.length/3;
      var entries = [];
      for ( var i = 0; i < historyCount; i++ ){
         var entry = { url:"", children:[], ID: 0};
         var index = i * 3;
         entry.url = historyData[index + 1];
         entry.title = unescape(historyData[index]);
         entry.scroll = historyData[index + 2];
         entries.push(entry);
      }
      return entries;
   }
}
